/****************************************************************************
 *  File Name:          AT94K_UTILS.H
 *  Title:              FPSLIC (AT94K) MACROS
 *  Version:            1.0
 *  Last Updated:       May 1, 2000
 *  Target Device:      AT94K10/20/40
 *
 *  Author:             Taylor Cox
 *  Personal Email:     tcox@atmel.com
 *  Support Email:      fpslic@atmel.com
 *  Support Hotline:    +1 (408) 436-4119 (USA)
 *
 *  Revision:           1.1.0
 *
 *  Overview:           Provides MACRO definitions for Bit Manipulation and
 *                      general Utilities
 *
 *  Function Summary:
 *
 *  Resources Used:     None
 *
 *  Notes:
 *      Revision 1.0.0 (TC)
 *          - Initial Release
 *      Revision 1.0.1 (TC)
 *          - added checkbit,setbit,clearbit macros
 *      Revision 1.0.2 (ian)
 *          - added SET_SIGNAL macro
 *      Revision 1.1.0 (TC)
 *          - Determined if IAR Compiler in Use
 *
 *
 *  Known Issues:
 *      Revision 1.0.0
 *          - None at this time
 ***************************************************************************/

#ifndef __at94k_utils_h
#define __at94k_utils_h

#include <ioat94k.h>

/* Determining Compiler */
/* Allows use of IAR Intrinsic Functions in EW v1.50B and 2.21B Beta */
#ifdef __IAR_SYSTEMS_ICC__
    #include <ina90.h>
#endif

/* Bit Manipulation */
#define CHECKBIT(ADDRESS, BIT)      ( (ADDRESS) &   ( 1 << (BIT) ) )
#define SETBIT(ADDRESS, BIT)        ( (ADDRESS) |=  ( 1 << (BIT) ) )
#define CLEARBIT(ADDRESS, BIT)      ( (ADDRESS) &= ~( 1 << (BIT) ) )
#define BIT(bitNumber)              ( 1 << (bitNumber) )

 /* Set FPGA<->I/O Select Lines 0-15 & send data over 8-Bit AVR<->FPGA
        internal data bus */
#define SET_SIGNAL(sigNo, data) \
    do { FISCR = ( (sigNo) % 4 );\
         *( (unsigned char *)0x34 + ( ((sigNo)>>2) & 3) ) = (data);\
       } while(0)

#endif
