library IEEE;
use	IEEE.STD_LOGIC_1164.all;
use	IEEE.STD_LOGIC_ARITH.all;
use	IEEE.STD_LOGIC_UNSIGNED.all;
library	WORK;
use	WORK.all;
entity	MESSAGE is
	generic(WIDTH : INTEGER := 8);
	port(
		DATA	: out	STD_LOGIC_VECTOR(WIDTH-1 downto 0);
		ENO	: out	STD_LOGIC;
		DBUS	: in	STD_LOGIC_VECTOR(7 downto 0);
		ENI	: in	STD_LOGIC;
		WRITE	: in	STD_LOGIC;
		RESET_Z	: in	STD_LOGIC;
		CLOCK	: in	STD_LOGIC
	);
end MESSAGE;

library IEEE;
use	IEEE.STD_LOGIC_1164.all;
use	IEEE.STD_LOGIC_ARITH.all;
use	IEEE.STD_LOGIC_UNSIGNED.all;
library	WORK;
use	WORK.all;
architecture RTL of MESSAGE is
begin

instance : process(CLOCK,RESET_Z,WRITE)
begin
	if(RESET_Z = '0') then
		DATA(WIDTH-1 downto 0)	<= (others => '0');
		ENO	<= '0';
	elsif(RISING_EDGE(CLOCK)) then
		if(WRITE = '1') then
			if(ENI = '1') then
				DATA(WIDTH-1 downto 0) <= DBUS(WIDTH-1 downto 0);
				ENO	<= '1';
			else
				ENO	<= '0';
			end if;
		end if;
	end if;
end process;

end RTL;



library	IEEE;
use	IEEE.STD_LOGIC_1164.all;
use	IEEE.STD_LOGIC_ARITH.all;
use	IEEE.STD_LOGIC_UNSIGNED.all;
library	WORK;
entity	PORTS	IS
	port	(
-- Output ports
		ABUS	: out	STD_LOGIC_VECTOR(23 downto 0);
		DBUS	: inout	STD_LOGIC_VECTOR(15 downto 0);
		RD_Z	: out	STD_LOGIC;	-- Read Strobe
		WRL_Z	: out	STD_LOGIC;	-- Lower Write Strobe
		WRH_Z	: out	STD_LOGIC;	-- Upper Write Strobe
		CS0_Z	: out	STD_LOGIC;	-- Chip Select Memory 0
		CS1_Z	: out	STD_LOGIC;	-- Chip Select Memory 1
		WAIT_Z	: in	STD_LOGIC;	-- Not yet implememted
		HOLD_Z	: in	STD_LOGIC;	-- Not yet implemented
		HLDA_Z	: out	STD_LOGIC;	-- Not yet implemented

-- AVR Interface
		ADIN	: in	STD_LOGIC_VECTOR(7 downto 0);
		ADOUT	: out	STD_LOGIC_VECTOR(7 downto 0);
		FIORE	: in	STD_LOGIC;	
		FIOWR	: in	STD_LOGIC;
		FIOSEL0	: in	STD_LOGIC;	-- Select Signal
		FIOSEL4	: in	STD_LOGIC;	-- Select Signal
		FIOSEL8	: in	STD_LOGIC;	-- Select Signal
		FIOSEL12: in	STD_LOGIC;	-- Select Signal
-- Common
		CLOCK	: in	STD_LOGIC;
		RESET_Z	: in	STD_LOGIC
	);
end PORTS;

-------------------------------------
library	IEEE;
use	IEEE.STD_LOGIC_1164.all;
use	IEEE.STD_LOGIC_ARITH.all;
use	IEEE.STD_LOGIC_UNSIGNED.all;
library	WORK;

architecture	RTL of PORTS is

component	MESSAGE
	generic(WIDTH : INTEGER := 8);
	port(
		DATA	: out	STD_LOGIC_VECTOR(WIDTH-1 downto 0);
		ENO	: out	STD_LOGIC;
		DBUS	: in	STD_LOGIC_VECTOR(7 downto 0);
		ENI	: in	STD_LOGIC;
		WRITE	: in	STD_LOGIC;
		RESET_Z	: in	STD_LOGIC;
		CLOCK	: in	STD_LOGIC
	);
end component;

signal	IABUS	: STD_LOGIC_VECTOR(23 downto 0);-- Internal Address Bus
signal	IDBUS	: STD_LOGIC_VECTOR(15 downto 0);-- Internal Data Bus
signal	CS	: STD_LOGIC_VECTOR(7 downto 0);	-- Chip Select
signal	ACS	: STD_LOGIC_VECTOR(3 downto 0);	-- Addresss chain Chip Select
signal	DCS	: STD_LOGIC_VECTOR(2 downto 0);	-- Data chain Chip Select
signal	CCS	: STD_LOGIC_VECTOR(1 downto 0);	-- Control chain Chip Select
signal	WRITE	: STD_LOGIC;

type	CTRL_REGISTER	is RECORD
	RD_Z	: STD_LOGIC;
	WRL_Z	: STD_LOGIC;
	WRH_Z	: STD_LOGIC;
	CS0_Z	: STD_LOGIC;
	CS1_Z	: STD_LOGIC;
	HLDA_Z	: STD_LOGIC;
	BUSOFF	: STD_LOGIC;
	UNUSED	: STD_LOGIC;
end record;

signal	CTRL	: CTRL_REGISTER;
signal	CTRLREG	: STD_LOGIC_VECTOR(7 downto 0);
TYPE	STATE_TYPE is (IDLE,FSMRD1,FSMRD2,FSMRD3,FSMWR1,FSMWR2,FSMWR3,FSMWR4,FSMWR5);
signal	STATE	: STATE_TYPE;

begin


-- AVR Read internal data
READIO	: process(CLOCK,RESET_Z,FIOSEL4,FIOSEL8,FIOSEL12,FIORE)
begin
	if(RESET_Z = '0') then
		ADOUT	<= (others => 'Z');
	elsif(FIORE = '1') then
		if(FIOSEL8 = '1') then			-- Lower data bus
			ADOUT	<= DBUS(7 downto 0);
		elsif(FIOSEL12 = '1') then		-- Upper databus
			ADOUT	<= DBUS(15 downto 8); 
		elsif(FIOSEL4 = '1') then		-- Control
			ADOUT	<= WAIT_Z & "000000" & HOLD_Z;
		else
			ADOUT	<= (others => 'Z');
		end if;
	end if;
end process;

-- Select address chain
CHAIN_SELECT	: process(CLOCK,RESET_Z,FIOSEL0,FIOWR)
begin
	if(RESET_Z = '0') then
			CS	<= (others => '0');
	elsif(RISING_EDGE(CLOCK)) then
		if((FIOSEL0 = '1') AND (FIOWR = '1'))then
			CS 	<= ADIN;
		elsif(WRITE = '1') then
			CS	<= (others => '0');
		end if;
	end if;
end process;

WRITE	<= '1' when ((FIOWR = '1') AND (FIOSEL4 = '1')) else '0';


-- The data chain is activated by bit 0
-- 0: D(07:00)
-- 1: D(15:08)
DCS(0)	<= CS(0);

-- The address chain is activated by bit 1, or by writing the data chain
-- 0: A(07:00)
-- 1: A(15:08)
-- 2: A(23:16)
ACS(0)	<= '1' when ((CS(1) = '1') OR  (DCS(2) = '1'))  else '0';

-- The control chain is written after the address chain 
-- 0: CTRL register
-- 1: Start access
CCS(0)	<= ACS(3);


D07_00 :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> IDBUS(7 downto 0),
		ENO	=> DCS(1),
		DBUS	=> ADIN,
		ENI	=> DCS(0),
		WRITE	=> WRITE,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);

D15_08 :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> IDBUS(15 downto 8),
		ENO	=> DCS(2),
		DBUS	=> ADIN,
		ENI	=> DCS(1),
		WRITE	=> WRITE,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);


A07_00 :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> IABUS(7 downto 0),
		ENO	=> ACS(1),
		DBUS	=> ADIN,
		ENI	=> ACS(0) ,
		WRITE	=> WRITE ,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);

A15_08 :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> IABUS(15 downto 8),
		ENO	=> ACS(2),
		DBUS	=> ADIN,
		ENI	=> ACS(1),
		WRITE	=> WRITE,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);

A23_16 :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> IABUS(23 downto 16),
		ENO	=> ACS(3),
		DBUS	=> ADIN,
		ENI	=> ACS(2),
		WRITE	=> WRITE,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);

CTRL_REG :	MESSAGE
	generic map(WIDTH => 8)
	port map(
		DATA	=> CTRLREG(7 downto 0),
		ENO	=> CCS(1),
		DBUS	=> ADIN,
		ENI	=> CCS(0),
		WRITE	=> WRITE,
		RESET_Z	=> RESET_Z,
		CLOCK	=> CLOCK
	);
CTRL.RD_Z	<= CTRLREG(0);
CTRL.WRL_Z	<= CTRLREG(1);
CTRL.WRH_Z	<= CTRLREG(2);
CTRL.CS0_Z	<= CTRLREG(3);
CTRL.CS1_Z	<= CTRLREG(4);
CTRL.HLDA_Z	<= CTRLREG(5);
CTRL.BUSOFF	<= CTRLREG(6);
CTRL.UNUSED	<= CTRLREG(7);

FSM	: process(RESET_Z,CLOCK,WRITE,CCS,CTRL,STATE)
begin
	if(RESET_Z = '0') then
		ABUS	<=	(others => 'Z');
		DBUS	<=	(others => 'Z');
		STATE	<=	IDLE;
		RD_Z	<=	'1';
		WRL_Z	<=	'1';
		WRH_Z	<=	'1';
		CS0_Z	<=	'1';
		CS1_Z	<=	'1';
	elsif(RISING_EDGE(CLOCK)) then
		case STATE is
		when IDLE	=>
			ABUS	<= IABUS;
			if((WRITE = '1') AND (CCS(1) = '1')) then	-- Start cycle
				if(CTRL.RD_Z = '1') then	-- Read cycle;
					STATE	<= FSMRD1;
				else				-- Write cycle
					STATE	<= FSMWR1;
					DBUS	<= IDBUS;
				end if;
				if(CTRL.CS0_Z = '1') then	-- Memory 0
					CS0_Z	<= '0';
				else				-- Memory 1
					CS1_Z	<= '0';
				end if;
			else
				STATE	<= IDLE;
				DBUS		<= (others => 'Z');
			end if;
		when	FSMRD1	=>
			RD_Z	<= '0';
			STATE	<= FSMRD2;
		when	FSMRD2	=>
			if((FIORE = '1') AND (FIOSEL12 = '1')) then
				STATE	<= FSMRD3;
			else
				STATE	<= FSMRD2;
			end if;
		when	FSMRD3	=>
				RD_Z	<= '1';
				CS0_Z	<= '1';
				CS1_Z	<= '1';
				STATE	<= IDLE;
		when	FSMWR1	=>
				WRL_Z	<= NOT CTRL.WRL_Z;
				WRH_Z	<= NOT CTRL.WRH_Z;
				STATE	<= FSMWR2;
		when	FSMWR2	=>
				STATE	<= FSMWR3;
		when	FSMWR3	=>
				STATE	<= FSMWR4;
		when	FSMWR4	=>
				STATE	<= FSMWR5;
				WRL_Z	<= '1';
				WRH_Z	<= '1';
		when	FSMWR5	=>
				STATE	<= IDLE;
				CS0_Z	<= '1';
				CS1_Z	<= '1';
				DBUS	<= (others	=> 'Z');
		when	others	=>
				STATE	<= IDLE;
				RD_Z	 <=	'1';
				WRL_Z	<=	'1';
				WRH_Z	<=	'1';
				CS0_Z	<=	'1';
				CS1_Z	<=	'1';
		end case;
	end if;
end process;
end RTL;

